
// ****************************************************************************
// *                                                                          *
// *    Type Definitions                                                      *
// *                                                                          *
// ****************************************************************************

#define byte unsigned char
#define word unsigned short
#define long unsigned int
#define real float

// ****************************************************************************
// *                                                                          *
// *    Special Purpose Register Access (SPR)                                 *
// *                                                                          *
// ****************************************************************************

#define PAR     ( *(          long * ) -0x1F0 )
#define CNT     ( *( volatile long * ) -0x1F1 )
#define INA     ( *( volatile long * ) -0x1F2 )
#define INB     ( *( volatile long * ) -0x1F3 )
#define OUTA    ( *(          long * ) -0x1F4 )
#define OUTB    ( *(          long * ) -0x1F5 )
#define DIRA    ( *(          long * ) -0x1F6 )
#define DIRB    ( *(          long * ) -0x1F7 )
#define CTRA    ( *( volatile long * ) -0x1F8 )
#define CTRB    ( *( volatile long * ) -0x1F9 )
#define FRQA    ( *( volatile long * ) -0x1FA )
#define FRQB    ( *( volatile long * ) -0x1FB )
#define PHSA    ( *( volatile long * ) -0x1FC )
#define PHSB    ( *( volatile long * ) -0x1FD )
#define VCFG    ( *(          long * ) -0x1FE )
#define VSCL    ( *(          long * ) -0x1FF )

// ****************************************************************************
// *                                                                          *
// *    Special Variable Access                                               *
// *                                                                          *
// ****************************************************************************

#define CLKFREQ ( *( volatile long * ) -0x200 )
#define CLKMODE ( *( volatile byte * ) -0x201 )
#define CHIPVER ( *(          byte * ) -0x202 )

// ****************************************************************************
// *                                                                          *
// *    In-Built Library Interfaces                                           *
// *                                                                          *
// ****************************************************************************

// .--------------------------------------------------------------------------.
//     Serial Communications                                                 
// `--------------------------------------------------------------------------'

extern void TxText( long pinNumber, int baudRate, char * strPtr );
extern void TxByte( long pinNumber, int baudRate, byte dataByte );
extern void TxRaw ( long pinNumber, int baudRate, byte dataByte );

extern long RxByte( long pinNumber, int baudRate );

// .--------------------------------------------------------------------------.
//     System Interface                                                      
// `--------------------------------------------------------------------------'

extern void WaitCnt( long cntMatch );
extern void WaitPeq( long pinMatch, long pinMask, long portNumber );
extern void WaitPne( long pinMatch, long pinMask, long portNumber );

// .--------------------------------------------------------------------------.
//     Task / Cog Control                                                           
// `--------------------------------------------------------------------------'

extern long CogId();
//extern long CogRun( void * funcAddr, long * stackBase, long stackSize );
extern void CogStop( long cogNumber );

extern long TaskId();
//extern long TaskInit( long taskNumber, void * funcAddr, long * stackBase, long stackSize );
//extern long TaskNew( void * funcAddr, long * stackBase, long stackSize );
extern void TaskStop( long taskNumber );

// .--------------------------------------------------------------------------.
//     Lock Control                                                          
// `--------------------------------------------------------------------------'

extern long LockNew();
extern long LockSet( long lockNumber );
extern long LockClr( long lockNumber );
extern void LockRet( long lockNumber );

// .--------------------------------------------------------------------------.
//     Memory Manipulation                                                   
// `--------------------------------------------------------------------------'

extern void ByteMove( byte * dstAdr, byte * srcAdr, long byteCount );
extern void WordMove( word * dstAdr, word * srcAdr, long wordCount );
extern void LongMove( long * dstAdr, long * srcAdr, long longCount );
extern void CharMove( char * dstAdr, char * srcAdr, long charCount );
extern void RealMove( real * dstAdr, real * srcAdr, long realCount );

extern void ByteFill( byte * dstAdr, long byteCount, byte byteValue );
extern void WordFill( word * dstAdr, long wordCount, word wordValue );
extern void LongFill( long * dstAdr, long longCount, long longValue );
extern void CharFill( char * dstAdr, long charCount, char charValue );
extern void RealFill( real * dstAdr, long realCount, real realValue );

// .--------------------------------------------------------------------------.
//     Integer Maths                                                         
// `--------------------------------------------------------------------------'

extern long MpyBig( long lhs, long rhs );
extern int  SMpyBig( int lhs, int rhs );

extern long Sqr( int number );

// .--------------------------------------------------------------------------.
//     Floating Point                                                        
// `--------------------------------------------------------------------------'

// ****************************************************************************
// *                                                                          *
// *    Helper Functions                                                      *
// *                                                                          *
// ****************************************************************************

extern void Asm( char * strPtr );

