
// ****************************************************************************
// *                                                                          *
// *    Type Definitions                                                      *
// *                                                                          *
// ****************************************************************************

#define byte unsigned char
#define word unsigned short
#define long unsigned int
#define real float

// ****************************************************************************
// *                                                                          *
// *    Special Purpose Register Access (SPR)                                 *
// *                                                                          *
// ****************************************************************************

#define PAR     ( *(          long * ) -0x1F0 )
#define CNT     ( *( volatile long * ) -0x1F1 )
#define INA     ( *( volatile long * ) -0x1F2 )
#define INB     ( *( volatile long * ) -0x1F3 )
#define OUTA    ( *(          long * ) -0x1F4 )
#define OUTB    ( *(          long * ) -0x1F5 )
#define DIRA    ( *(          long * ) -0x1F6 )
#define DIRB    ( *(          long * ) -0x1F7 )
#define CTRA    ( *( volatile long * ) -0x1F8 )
#define CTRB    ( *( volatile long * ) -0x1F9 )
#define FRQA    ( *( volatile long * ) -0x1FA )
#define FRQB    ( *( volatile long * ) -0x1FB )
#define PHSA    ( *( volatile long * ) -0x1FC )
#define PHSB    ( *( volatile long * ) -0x1FD )
#define VCFG    ( *(          long * ) -0x1FE )
#define VSCL    ( *(          long * ) -0x1FF )

// ****************************************************************************
// *                                                                          *
// *    Special Variable Access                                               *
// *                                                                          *
// ****************************************************************************

#define CLKFREQ ( *( volatile long * ) -0x200 )
#define CLKMODE ( *( volatile byte * ) -0x201 )
#define CHIPVER ( *(          byte * ) -0x202 )

// ****************************************************************************
// *                                                                          *
// *    In-Built Library Interfaces                                           *
// *                                                                          *
// ****************************************************************************

// .--------------------------------------------------------------------------.
//     Serial Communications                                                 
// `--------------------------------------------------------------------------'

extern void TxText( long pinNumber, int baudRate, char * strPtr );
extern void TxByte( long pinNumber, int baudRate, byte dataByte );
extern void TxRaw ( long pinNumber, int baudRate, byte dataByte );

extern long RxByte( long pinNumber, int baudRate );

// .--------------------------------------------------------------------------.
//     System Interface                                                      
// `--------------------------------------------------------------------------'

extern void WaitCnt( long cntMatch );
extern void WaitPeq( long pinMatch, long pinMask, long portNumber );
extern void WaitPne( long pinMatch, long pinMask, long portNumber );

// .--------------------------------------------------------------------------.
//     Task / Cog Control                                                           
// `--------------------------------------------------------------------------'

extern long CogId();
//extern long CogRun( void * funcAddr, long * stackBase, long stackSize );
extern void CogStop( long cogNumber );

extern long TaskId();
//extern long TaskInit( long taskNumber, void * funcAddr, long * stackBase, long stackSize );
//extern long TaskNew( void * funcAddr, long * stackBase, long stackSize );
extern void TaskStop( long taskNumber );

// .--------------------------------------------------------------------------.
//     Lock Control                                                          
// `--------------------------------------------------------------------------'

extern long LockNew();
extern long LockSet( long lockNumber );
extern long LockClr( long lockNumber );
extern void LockRet( long lockNumber );

// .--------------------------------------------------------------------------.
//     Memory Manipulation                                                   
// `--------------------------------------------------------------------------'

extern void ByteMove( byte * dstAdr, byte * srcAdr, long byteCount );
extern void WordMove( word * dstAdr, word * srcAdr, long wordCount );
extern void LongMove( long * dstAdr, long * srcAdr, long longCount );
extern void CharMove( char * dstAdr, char * srcAdr, long charCount );
extern void RealMove( real * dstAdr, real * srcAdr, long realCount );

extern void ByteFill( byte * dstAdr, long byteCount, byte byteValue );
extern void WordFill( word * dstAdr, long wordCount, word wordValue );
extern void LongFill( long * dstAdr, long longCount, long longValue );
extern void CharFill( char * dstAdr, long charCount, char charValue );
extern void RealFill( real * dstAdr, long realCount, real realValue );

// .--------------------------------------------------------------------------.
//     Integer Maths                                                         
// `--------------------------------------------------------------------------'

extern long MpyBig( long lhs, long rhs );
extern int  SMpyBig( int lhs, int rhs );

extern long Sqr( int number );

// .--------------------------------------------------------------------------.
//     Floating Point                                                        
// `--------------------------------------------------------------------------'

// ****************************************************************************
// *                                                                          *
// *    Helper Functions                                                      *
// *                                                                          *
// ****************************************************************************

extern void Asm( char * strPtr );


/*****************************************************************************************
Code specific 
*/

    #define RXPIN     31
    #define TXPIN     30
    #define BAUD    9600
    #define INVERT     0


/********************************************************************************************
Global variables
*/
	long stack[30];   // Revisit this when/if Hippy we get more info on cognew 
    char buf[buflen]; //Text Buffer max length
    int cx;           //char from input stream
	int pb,po;        //Text Buffer pointer, TV offset pointer
	char cc;          //character hidden by cursor
    int id = 0;       //tv cog #
	int z,j;          //Keyboard Buffer input pointer, output pointer 
   	char cmdbuf[20];  //Holds command string
	char kb[64];      //Keyboard Buffer
		 
/***************************************************************************************/			         
prototype........provide documentation here DONT PUT IT OFF....to me 
   
    void getcx(); 

    void keyin();

    void tvText_out(unsigned int);

    void tvText_stop(void)

    int tvText_start(int basepin)

    void tvText_setColorPalette(char* ptr);

    void tvText_dec(int value);

    void tvText_str(char* sptr);

    void cmd();

    void screen_update()  

/***************************************************************************************
LIB routines would be required to support this code. If keyin, combined with 
Native code Libs routines for Wordmove, Bytemove Wordfill and Bytefill then my Retx 
method would Fly.   
*/

int    Cognew(arg1, agr2);         /* needed to load keyin,  Hippy  ??  */

int    CognewNative(arg1, arg2);   /* needed to load binary, Hippy ??   */

void   Cogstop(arg1);              /*                        Hippy ??   */

void   wordfill(arg1,arg2,arg3);   /* can work around but very slow        */

void   bytemove(arg1,arg2,arg3);   /* have done a work around but SLOW see retx method */

void   ASM(arg1,arg2, etc)         /*                         Hippy ??   */ 


/*****************************************************************************************
Start of MAIN */

void main()                             
{    j=0;                              // init getcx pointer
     z=0;                              // init keyin pointer
	 tvText_start(12);                 // start tv at basepin
	 cognew(keyin,&stack[18]);         // CogNew is still to be implemented
   	 while(1)                          // go for it
	   { getcx();                         // get next char in keyboard buffer 
	   if (cx!=0x0) { tvText_out(cx); }   // clear screen not allowed here
       }
}

/******************************************************************************************
start Tv in its own cog */

int tvText_start(int basepin)          
 {  int i;
    gTvText.status = 0;          
    gTvText.enable = 1;
    gTvText.pins   = ((basepin & 0x38) << 1) | (((basepin & 4) == 4) ? 0b101 : 0); 
    gTvText.mode   = 0b10010;
    gTvText.screen = gscreen;    
    gTvText.colors = gcolors;
    gTvText.ht = TV_TEXT_COLS;
    gTvText.vt = TV_TEXT_ROWS;
    gTvText.hx = 4;
    gTvText.vx = 1;
    gTvText.ho = 0;
    gTvText.vo = 0;
    gTvText.broadcast = 0;
    gTvText.auralcog  = 0;
    id = cognewNative(TV_Text_Array, &gTvText);                          //NOT IMPLEMENTED ATM
    gTvTextCog = id+1;
    tvText_setColorPalette(&gpalette[TV_TEXT_PAL_WHITE_BLUE]);
    wordfill(&gscreen[0], 0x220, TV_TEXT_SCREENSIZE);                    //NOT IMPLEMENTED ATM   
	for(i=0; i<=buflen; i++) { buf[i]=0x20;}
    waitcnt(CLKFREQ/8000 + CNT);;                    
    return id;
  }
	

/*******************************************************************************************************
stops the cog running the native assembly driver. 
Notes to myself.
Need to add Binary Tv_Text_Array.h file to Zip folder. 
Check with Steve to see if he has a DOS bat file to generate these files.
Can use cygwin bash shell for now. Ask why he changes the starting point. Me thinks is the same for each binary ????

  
*/

void tvText_stop(void) 
   { int id = gTvTextCog - 1;
     if(gTvTextCog > 0) {
        asm("cogstop %id");           //NOT IMPLEMENTED ATM                 
     }
   }

/*
TV_Text setcolors function sets the palette to that defined by pointer.
See header file for more details. */
 
void tvText_setColorPalette(char* ptr)
{   int  ii = 0;
    int  mm = 0;
    int  fg = 0;
    int  bg = 0;
    for(ii = 0; ii < TV_TEXT_COLORTABLE_SIZE; ii += 2)
      { mm = ii + 1; 
        fg = ptr[ii] << 0;
        bg = ptr[mm] << 0;
        gcolors[ii]  = fg << 24 | bg << 16 | fg << 8 | bg;
        gcolors[mm]  = fg << 24 | fg << 16 | bg << 8 | bg;
      }        
}


void tvText_dec(int value)
 {  char b[128];
    itoa(b, value, 10);
    tvText_str(b);
 }

void tvText_str(char* sptr)
{   
    int l,k;
    l=(TV_TEXT_SCREENSIZE-39);
    while(*sptr) {
        k =(*(sptr++));
		gscreen[l++]=((2<< 1 | k & 1) << 10) + (0x200 + (k & 0xfe));
    }
}


/* the CMD> method is experimental at this stage, but it is envisaged that is will do an auto save 
of the text buffer to SDcard and provide users with access to other user function. 
*/


void cmd()                         
 {
	 int m = 0 ;
	 int i = 0;
	 int l = TV_TEXT_SCREENSIZE-30;
	 short int ix; 
     for(ix=(TV_TEXT_SCREENSIZE-39) ; ix<=(TV_TEXT_SCREENSIZE-2) ;  ix++)
     {  gscreen[ix]=((2 << 1 | SPACE & 1) << 10) + (0x200 + (SPACE & 0xfe)); }
	 tvText_str("CMD>");
	 while(cx != RETN)
       { waitcnt(CLKFREQ/120+CNT)            
	     getcx();
	     if (i<=20)
	     {	if (cx!=0x0)
	    	{ gscreen[l++]=((2<< 1 | cx & 1) << 10) + (0x200 + (cx & 0xfe));
	          cmdbuf[i++] = cx; }
	     }
	    }
    /* execute cmd string here or return to editor	*/ 
   }
	
	
	
	
void getcx()                     /* unload keyboard buffer and decode arrow key for my tiny Prop WP */
{
if(j!=z)                                       /* is buffer empty ??   'z' is input pointer in Keyin */
	 {
	 j++;
 	 if (j>=60){j=0;}                          /* rewind buffer 'j' output pointer  */
			cx =kb[j];                         /* get next char in buffer           */
			  if (cx==0x1b)                    /* look for arrow key sequence       */
			      {
				  j++;
				  if (j>=60){j=0;}   
				  waitcnt(CLKFREQ/200 + CNT);  /* don't check too early  */ 
		       	  if (kb[j]== 0x5b )           /* check next key in sequence */
	              		{
						j++;
						if (j>=60){j=0;}   
				        cx= 0x80 + kb[j];      /* found it, now modify for my tvText_out  */
						return;                /* return with arrow code                  */ 
				        }
					cx=0x0;	                   /* not decoded hook for other sequences    */
					return;
				  } 		
			return;                           /* normal return                            */	    
     }
	cx=0x0;	                                  /* buffer empty                             */
	return; 	 
}


void keyin()            /* buffer RxByte required to trap ESC keyboard Sequences (repeated arrow keys)   */ 
  {
  while (1)
     {  cx = RxByte(RXPIN,BAUD) ;
        if (cx!=0x00)
        { z++;	
 	      if (z>=60){z=0;}              /* rewind pointer  */
	      kb[z]=cx;  
        }   	
      }
   }

void    tvText_out(int c)             
   {switch(c)
       {   case cls:
                wordfill(&gscreen[0], color << 11 |0x220, TV_TEXT_SCREENSIZE);   /*NOT IMPLEMENTED ATM */
                po= 0;
                pb = 0;
                break;
            case home:
                po = 0;
                pb = 0;
                break;
            case esc:  
			    cmd();
			    break;  
            case tab:         /* fall though ATM  */
            case 0x0A:        /* fall though ATM  */
            case BS:          /* fall though ATM  */ 
	            if (pb >= 0)
			  { buf[pb] = 0x20;
			    pb--;
			    cc = buf[pb];
			    buf[pb] = '_'; }
			    break;	
            case 0x0c:        /* fall though ATM   */ 
            case RETN:  
	            retx();	
				break;	
			case up:
			if (pb >= TV_TEXT_COLS)
	        {  if ((pb - po) <= TV_TEXT_COLS) { po -= TV_TEXT_COLS;}
 			   else
			{  buf[pb]=cc;
			   pb -= TV_TEXT_COLS;
			   cc = buf[pb];
			   buf[pb]='_';  }
			}	
			break; 	 
			case down:
			if (pb <= buflen)
			{
			if (pb - po > TV_TEXT_SCREENSIZE) { po += TV_TEXT_COLS; }
			else
		{
			buf[pb] = cc;
			pb += TV_TEXT_COLS;
			cc = buf[pb];
			buf[pb] = '_';
			}
			}
			break;
			case right:
			if (pb <= buflen)
			{
			buf[pb] = cc;
			  pb++;
			  cc = buf[pb];
			  buf[pb] = '_';
			} 
			break;  
			case left:
			if (pb >= 0)
			  {
			  buf[pb] = cc;
			  pb--;
			  cc = buf[pb];
			  buf[pb] = '_';
			  }
			break;  
			default:	
			buf[pb] = c;   	   				 //save new character in buffer
			pb++;           		         //inc pointer
			cc = buf[pb];  			  		 //save current cursor position
			buf[pb] ='_';      		   		 //write new cursor
		 }     
         screen_update();                    //update screen ..keep in sync
}        

/* keep text buffer in snyc with gscreen .......re visit this. Bug when row=col= 0 , therefore there may be others */

void screen_update()
   { int i;
	 if ((pb-po)>=lastrow){	 po += 40;}       
	 for (i=0; i<=lastrow  ; i++ ) {gscreen[i]=((color << 1 | buf[po + i] & 1) << 10) + (0x200 + (buf[po + i] & 0xfe)); }	 
   }
	 
/* The retx method below needs bytemove and bytefill, so I will leave it for now and see what 
Native code Libs AIChip comes up with. This work around will be too slow */	 


void retx()                             
{	 									 
    int v,k,j,i,temp;					 
	v =1+(pb/TV_TEXT_COLS);             //calculate number of lines in buffer include part line 
	v = v*40;
    k = (pb % TV_TEXT_COLS);            //calculate number on characters in current line
	buf[pb]=cc; 
	/***
	**move all chars below this line down 1 line to make room for newline
	*/ 
	for (i=1; i<=TV_TEXT_COLS; i++)
	    {
	     for (j=buflen; j>=v; j--){buf[j]= buf[j-1]; }    
		}
	/***
	*clear the line below after move
	*/  
	for (i=0; i<=TV_TEXT_COLS-1; i++){buf[(v)+i]= SPACE;}            
    /***
	*move chars from cursor to EOL to line below
	*/
	for (i= 0;i <= TV_TEXT_COLS-(k+1) ;i++)  {buf[(v)+i]= buf[pb+i];} 
	
	/***
	*now to clear from cursor clear to EOL  
	*/
    for (i = 0;i <= TV_TEXT_COLS-(k+1) ;i++) {buf[(pb)+i]= SPACE;} 
	/***
	*now reset cursor position 
	*/
	pb=v;                                
	buf[pb] ='_';
	}
	 